#pragma once

#include <string>
#include <variant>
#include <functional>
#include <optional>

#include "core/type_list.h"
#include "request/session_share.h"

namespace request
{
namespace req_opt
{
#define REQ_OPT_PROP(Type, Name, Init)    \
    Type Name Init;                       \
    auto&     set_##Name(const Type& v) { \
        Name = v;                     \
        return *this;                 \
    }

struct Timeout {
    REQ_OPT_PROP(i64, low_speed, {})
    REQ_OPT_PROP(i64, connect_timeout, {})
    REQ_OPT_PROP(i64, transfer_timeout, {})
};

struct Proxy {
    enum class Type
    {
        HTTP    = 0,
        HTTPS2  = 3,
        SOCKS4  = 4,
        SOCKS5  = 5,
        SOCKS4A = 6,
        SOCKS5H = 7
    };
    REQ_OPT_PROP(Type, type, { Type::HTTP })
    REQ_OPT_PROP(std::string, content, {})
};

struct Tcp {
    REQ_OPT_PROP(bool, keepalive, {})
    REQ_OPT_PROP(i64, keepidle, {})
    REQ_OPT_PROP(i64, keepintvl, {})
};

struct SSL {
    REQ_OPT_PROP(bool, verify_certificate, { true })
};

struct Read {
    using Callback = std::function<usize(byte* ptr, usize size)>;
    REQ_OPT_PROP(Callback, callback, {})
    REQ_OPT_PROP(usize, size, { 0 })
};

struct Share {
    REQ_OPT_PROP(std::optional<SessionShare>, share, {})
};

#undef REQ_OPT_PROP

using opts = ycore::type_list<Timeout, Proxy, Tcp, SSL, Read, Share>;

} // namespace req_opt

using RequestOpts = req_opt::opts;
using RequestOpt  = RequestOpts::to<std::variant>;

} // namespace request