
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <QObject>

#include "qcm_interface/model.h"


namespace qcm::model { class AppInfo; }
namespace qcm::model {

template<>
class Model<qcm::model::AppInfo>::Private {
public:
QString id;
QString name;
QString version;
QString author;
QString summary;
};


// Helper
template<>
struct Model<qcm::model::AppInfo>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->id)>) { if(!(self.d_func()->id == m.d_func()->id)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->name)>) { if(!(self.d_func()->name == m.d_func()->name)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->version)>) { if(!(self.d_func()->version == m.d_func()->version)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->author)>) { if(!(self.d_func()->author == m.d_func()->author)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->summary)>) { if(!(self.d_func()->summary == m.d_func()->summary)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::AppInfo>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::AppInfo>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::AppInfo>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::AppInfo>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::AppInfo>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::AppInfo>::from_json(const json::njson& j) {
j.at("id"sv).get_to(as_ref(m_ptr->id));
j.at("name"sv).get_to(as_ref(m_ptr->name));
j.at("version"sv).get_to(as_ref(m_ptr->version));
j.at("author"sv).get_to(as_ref(m_ptr->author));
j.at("summary"sv).get_to(as_ref(m_ptr->summary));
}

template<>
void Model<qcm::model::AppInfo>::to_json(json::njson& j) const {
j["id"sv] = as_ref(m_ptr->id);
j["name"sv] = as_ref(m_ptr->name);
j["version"sv] = as_ref(m_ptr->version);
j["author"sv] = as_ref(m_ptr->author);
j["summary"sv] = as_ref(m_ptr->summary);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/app_info.h"

namespace qcm::model {

auto AppInfo::id() const -> to_param<QString> {
    auto d = qcm::model::Model<AppInfo>::d_func();
    return d->id;
}

void AppInfo::set_id(to_param<QString> val) {
    auto d = qcm::model::Model<AppInfo>::d_func();
    auto& p = d->id;
    if(val != p) {
        p = val;
        
    }
}


auto AppInfo::name() const -> to_param<QString> {
    auto d = qcm::model::Model<AppInfo>::d_func();
    return d->name;
}

void AppInfo::set_name(to_param<QString> val) {
    auto d = qcm::model::Model<AppInfo>::d_func();
    auto& p = d->name;
    if(val != p) {
        p = val;
        
    }
}


auto AppInfo::version() const -> to_param<QString> {
    auto d = qcm::model::Model<AppInfo>::d_func();
    return d->version;
}

void AppInfo::set_version(to_param<QString> val) {
    auto d = qcm::model::Model<AppInfo>::d_func();
    auto& p = d->version;
    if(val != p) {
        p = val;
        
    }
}


auto AppInfo::author() const -> to_param<QString> {
    auto d = qcm::model::Model<AppInfo>::d_func();
    return d->author;
}

void AppInfo::set_author(to_param<QString> val) {
    auto d = qcm::model::Model<AppInfo>::d_func();
    auto& p = d->author;
    if(val != p) {
        p = val;
        
    }
}


auto AppInfo::summary() const -> to_param<QString> {
    auto d = qcm::model::Model<AppInfo>::d_func();
    return d->summary;
}

void AppInfo::set_summary(to_param<QString> val) {
    auto d = qcm::model::Model<AppInfo>::d_func();
    auto& p = d->summary;
    if(val != p) {
        p = val;
        
    }
}

}
