
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <QObject>

#include "qcm_interface/model.h"

#include "core/type_list.h"


namespace qcm::model { class Page; }
namespace qcm::model {

template<>
class Model<qcm::model::Page>::Private {
public:
QString name;
QString icon;
QString source;
bool cache;
bool primary;
};


// Helper
template<>
struct Model<qcm::model::Page>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->name)>) { if(!(self.d_func()->name == m.d_func()->name)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->icon)>) { if(!(self.d_func()->icon == m.d_func()->icon)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->source)>) { if(!(self.d_func()->source == m.d_func()->source)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->cache)>) { if(!(self.d_func()->cache == m.d_func()->cache)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->primary)>) { if(!(self.d_func()->primary == m.d_func()->primary)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::Page>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::Page>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::Page>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::Page>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::Page>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::Page>::from_json(const json::njson& j) {
j.at("name"sv).get_to(as_ref(m_ptr->name));
j.at("icon"sv).get_to(as_ref(m_ptr->icon));
j.at("source"sv).get_to(as_ref(m_ptr->source));
j.at("cache"sv).get_to(as_ref(m_ptr->cache));
j.at("primary"sv).get_to(as_ref(m_ptr->primary));
}

template<>
void Model<qcm::model::Page>::to_json(json::njson& j) const {
j["name"sv] = as_ref(m_ptr->name);
j["icon"sv] = as_ref(m_ptr->icon);
j["source"sv] = as_ref(m_ptr->source);
j["cache"sv] = as_ref(m_ptr->cache);
j["primary"sv] = as_ref(m_ptr->primary);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/page.h"

namespace qcm::model {

auto Page::name() const -> to_param<QString> {
    auto d = qcm::model::Model<Page>::d_func();
    return d->name;
}

void Page::set_name(to_param<QString> val) {
    auto d = qcm::model::Model<Page>::d_func();
    auto& p = d->name;
    if(val != p) {
        p = val;
        
    }
}


auto Page::icon() const -> to_param<QString> {
    auto d = qcm::model::Model<Page>::d_func();
    return d->icon;
}

void Page::set_icon(to_param<QString> val) {
    auto d = qcm::model::Model<Page>::d_func();
    auto& p = d->icon;
    if(val != p) {
        p = val;
        
    }
}


auto Page::source() const -> to_param<QString> {
    auto d = qcm::model::Model<Page>::d_func();
    return d->source;
}

void Page::set_source(to_param<QString> val) {
    auto d = qcm::model::Model<Page>::d_func();
    auto& p = d->source;
    if(val != p) {
        p = val;
        
    }
}


auto Page::cache() const -> to_param<bool> {
    auto d = qcm::model::Model<Page>::d_func();
    return d->cache;
}

void Page::set_cache(to_param<bool> val) {
    auto d = qcm::model::Model<Page>::d_func();
    auto& p = d->cache;
    if(val != p) {
        p = val;
        
    }
}


auto Page::primary() const -> to_param<bool> {
    auto d = qcm::model::Model<Page>::d_func();
    return d->primary;
}

void Page::set_primary(to_param<bool> val) {
    auto d = qcm::model::Model<Page>::d_func();
    auto& p = d->primary;
    if(val != p) {
        p = val;
        
    }
}

}
