
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <QObject>

#include "qcm_interface/model.h"


namespace qcm::model { class PluginInfo; }
namespace qcm::model {

template<>
class Model<qcm::model::PluginInfo>::Private {
public:
QString name;
QString fullname;
QUrl icon;
};


// Helper
template<>
struct Model<qcm::model::PluginInfo>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->name)>) { if(!(self.d_func()->name == m.d_func()->name)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->fullname)>) { if(!(self.d_func()->fullname == m.d_func()->fullname)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->icon)>) { if(!(self.d_func()->icon == m.d_func()->icon)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::PluginInfo>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::PluginInfo>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::PluginInfo>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::PluginInfo>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::PluginInfo>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::PluginInfo>::from_json(const json::njson& j) {
j.at("name"sv).get_to(as_ref(m_ptr->name));
j.at("fullname"sv).get_to(as_ref(m_ptr->fullname));
j.at("icon"sv).get_to(as_ref(m_ptr->icon));
}

template<>
void Model<qcm::model::PluginInfo>::to_json(json::njson& j) const {
j["name"sv] = as_ref(m_ptr->name);
j["fullname"sv] = as_ref(m_ptr->fullname);
j["icon"sv] = as_ref(m_ptr->icon);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/plugin_info.h"

namespace qcm::model {

auto PluginInfo::name() const -> to_param<QString> {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    return d->name;
}

void PluginInfo::set_name(to_param<QString> val) {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    auto& p = d->name;
    if(val != p) {
        p = val;
        
    }
}


auto PluginInfo::fullname() const -> to_param<QString> {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    return d->fullname;
}

void PluginInfo::set_fullname(to_param<QString> val) {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    auto& p = d->fullname;
    if(val != p) {
        p = val;
        
    }
}


auto PluginInfo::icon() const -> to_param<QUrl> {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    return d->icon;
}

void PluginInfo::set_icon(to_param<QUrl> val) {
    auto d = qcm::model::Model<PluginInfo>::d_func();
    auto& p = d->icon;
    if(val != p) {
        p = val;
        
    }
}

}
