
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <QObject>

#include "qcm_interface/model.h"


namespace qcm::model { class RouteMsg; }
namespace qcm::model {

template<>
class Model<qcm::model::RouteMsg>::Private {
public:
QUrl url;
QVariantMap props;
};


// Helper
template<>
struct Model<qcm::model::RouteMsg>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->url)>) { if(!(self.d_func()->url == m.d_func()->url)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->props)>) { if(!(self.d_func()->props == m.d_func()->props)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::RouteMsg>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::RouteMsg>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::RouteMsg>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::RouteMsg>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::RouteMsg>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::RouteMsg>::from_json(const json::njson& j) {
j.at("url"sv).get_to(as_ref(m_ptr->url));
j.at("props"sv).get_to(as_ref(m_ptr->props));
}

template<>
void Model<qcm::model::RouteMsg>::to_json(json::njson& j) const {
j["url"sv] = as_ref(m_ptr->url);
j["props"sv] = as_ref(m_ptr->props);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/router_msg.h"

namespace qcm::model {

auto RouteMsg::url() const -> to_param<QUrl> {
    auto d = qcm::model::Model<RouteMsg>::d_func();
    return d->url;
}

void RouteMsg::set_url(to_param<QUrl> val) {
    auto d = qcm::model::Model<RouteMsg>::d_func();
    auto& p = d->url;
    if(val != p) {
        p = val;
        
    }
}


auto RouteMsg::props() const -> to_param<QVariantMap> {
    auto d = qcm::model::Model<RouteMsg>::d_func();
    return d->props;
}

void RouteMsg::set_props(to_param<QVariantMap> val) {
    auto d = qcm::model::Model<RouteMsg>::d_func();
    auto& p = d->props;
    if(val != p) {
        p = val;
        
    }
}

}
