
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <QObject>

#include "qcm_interface/model.h"

#include "qcm_interface/model/user_model.h"

#include "qcm_interface/model/page.h"

#include "qcm_interface/client.h"


namespace qcm::model { class Session; }
namespace qcm::model {

template<>
class Model<qcm::model::Session, QObject>::Private {
public:
qcm::model::UserAccount* user;
std::vector<Page> pages;
bool valid;
QString provider;
bool supportComment;
};


// Helper
template<>
struct Model<qcm::model::Session, QObject>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->user)>) { if(!(self.d_func()->user == m.d_func()->user)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->pages)>) { if(!(self.d_func()->pages == m.d_func()->pages)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->valid)>) { if(!(self.d_func()->valid == m.d_func()->valid)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->provider)>) { if(!(self.d_func()->provider == m.d_func()->provider)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->supportComment)>) { if(!(self.d_func()->supportComment == m.d_func()->supportComment)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::Session, QObject>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::Session, QObject>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::Session, QObject>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::Session, QObject>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::Session, QObject>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::Session, QObject>::from_json(const json::njson& j) {
j.at("user"sv).get_to(as_ref(m_ptr->user));
j.at("pages"sv).get_to(as_ref(m_ptr->pages));
j.at("valid"sv).get_to(as_ref(m_ptr->valid));
j.at("provider"sv).get_to(as_ref(m_ptr->provider));
j.at("supportComment"sv).get_to(as_ref(m_ptr->supportComment));
}

template<>
void Model<qcm::model::Session, QObject>::to_json(json::njson& j) const {
j["user"sv] = as_ref(m_ptr->user);
j["pages"sv] = as_ref(m_ptr->pages);
j["valid"sv] = as_ref(m_ptr->valid);
j["provider"sv] = as_ref(m_ptr->provider);
j["supportComment"sv] = as_ref(m_ptr->supportComment);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/session.h"

namespace qcm::model {

auto Session::user() const -> to_param<qcm::model::UserAccount*> {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    return d->user;
}

void Session::set_user(to_param<qcm::model::UserAccount*> val) {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    auto& p = d->user;
    if(val != p) {
        p = val;
        Q_EMIT userChanged();
    }
}


auto Session::pages() const -> to_param<std::vector<Page>> {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    return d->pages;
}

void Session::set_pages(to_param<std::vector<Page>> val) {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    auto& p = d->pages;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto Session::valid() const -> to_param<bool> {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    return d->valid;
}

void Session::set_valid(to_param<bool> val) {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    auto& p = d->valid;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto Session::provider() const -> to_param<QString> {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    return d->provider;
}

void Session::set_provider(to_param<QString> val) {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    auto& p = d->provider;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto Session::supportComment() const -> to_param<bool> {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    return d->supportComment;
}

void Session::set_supportComment(to_param<bool> val) {
    auto d = qcm::model::Model<Session, QObject>::d_func();
    auto& p = d->supportComment;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}

}
