
#include "qcm_interface/model.h"
#include "json_helper/helper.inl"
#include <map>

#include <QObject>

#include <QPointer>

#include "qcm_interface/model.h"

#include "core/log.h"


namespace qcm::model { class UserAccount; }
namespace qcm::model {

template<>
class Model<qcm::model::UserAccount, QObject>::Private {
public:
qcm::model::ItemId userId;
QString token;
QString nickname;
QString avatarUrl;
QUrl server;
QUrl session_file;
Extra extra;
};


// Helper
template<>
struct Model<qcm::model::UserAccount, QObject>::Helper {
    template<bool B, typename T>
    static void copy_assign(T& self, const T& m) {
        if constexpr(B) {
            *(self.m_ptr) = *(m.m_ptr);
        }
    }
    template<bool B, typename T>
    static void move_assign(T& self, T&& m) {
        if constexpr(B) {
            std::swap(self.m_ptr, m.m_ptr);
        }
    }
    template<typename T>
    static bool partial_equal(const T& self, const T& m) {
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->userId)>) { if(!(self.d_func()->userId == m.d_func()->userId)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->token)>) { if(!(self.d_func()->token == m.d_func()->token)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->nickname)>) { if(!(self.d_func()->nickname == m.d_func()->nickname)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->avatarUrl)>) { if(!(self.d_func()->avatarUrl == m.d_func()->avatarUrl)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->server)>) { if(!(self.d_func()->server == m.d_func()->server)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->session_file)>) { if(!(self.d_func()->session_file == m.d_func()->session_file)) return false; }
        if constexpr(ycore::has_equal_operator<decltype(self.d_func()->extra)>) { if(!(self.d_func()->extra == m.d_func()->extra)) return false; }
        return true;
    }
};


template<>
C_DECL_EXPORT Model<qcm::model::UserAccount, QObject>::Model() : m_ptr(make_up<Private>()) {}

template<>
C_DECL_EXPORT Model<qcm::model::UserAccount, QObject>::~Model() {}


template<>
C_DECL_EXPORT void Model<qcm::model::UserAccount, QObject>::assign(const Model& m) {
    Helper::copy_assign<std::is_move_assignable_v<Base> && !false>(*this, m);
}
template<>
C_DECL_EXPORT void Model<qcm::model::UserAccount, QObject>::assign(Model&& m) noexcept {
    Helper::move_assign<std::is_move_assignable_v<Base> && !false>(*this, std::move(m));
}
template<>
C_DECL_EXPORT bool Model<qcm::model::UserAccount, QObject>::operator==(const Model& m) const {
    return Helper::partial_equal(*this, m);
}


template<>
void Model<qcm::model::UserAccount, QObject>::from_json(const json::njson& j) {
j.at("userId"sv).get_to(as_ref(m_ptr->userId));
j.at("token"sv).get_to(as_ref(m_ptr->token));
j.at("nickname"sv).get_to(as_ref(m_ptr->nickname));
j.at("avatarUrl"sv).get_to(as_ref(m_ptr->avatarUrl));
j.at("server"sv).get_to(as_ref(m_ptr->server));
j.at("session_file"sv).get_to(as_ref(m_ptr->session_file));
j.at("extra"sv).get_to(as_ref(m_ptr->extra));
}

template<>
void Model<qcm::model::UserAccount, QObject>::to_json(json::njson& j) const {
j["userId"sv] = as_ref(m_ptr->userId);
j["token"sv] = as_ref(m_ptr->token);
j["nickname"sv] = as_ref(m_ptr->nickname);
j["avatarUrl"sv] = as_ref(m_ptr->avatarUrl);
j["server"sv] = as_ref(m_ptr->server);
j["session_file"sv] = as_ref(m_ptr->session_file);
j["extra"sv] = as_ref(m_ptr->extra);
}



}



#include "/build/qcm-git/src/Qcm/qcm_interface/include/qcm_interface/model/user_account.h"

namespace qcm::model {

auto UserAccount::userId() const -> to_param<qcm::model::ItemId> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->userId;
}

void UserAccount::set_userId(to_param<qcm::model::ItemId> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->userId;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::token() const -> to_param<QString> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->token;
}

void UserAccount::set_token(to_param<QString> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->token;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::nickname() const -> to_param<QString> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->nickname;
}

void UserAccount::set_nickname(to_param<QString> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->nickname;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::avatarUrl() const -> to_param<QString> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->avatarUrl;
}

void UserAccount::set_avatarUrl(to_param<QString> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->avatarUrl;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::server() const -> to_param<QUrl> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->server;
}

void UserAccount::set_server(to_param<QUrl> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->server;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::session_file() const -> to_param<QUrl> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->session_file;
}

void UserAccount::set_session_file(to_param<QUrl> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->session_file;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}


auto UserAccount::extra() const -> to_param<Extra> {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    return d->extra;
}

void UserAccount::set_extra(to_param<Extra> val) {
    auto d = qcm::model::Model<UserAccount, QObject>::d_func();
    auto& p = d->extra;
    if(val != p) {
        p = val;
        Q_EMIT infoChanged();
    }
}

}
